# Author-AndyB
# Description-Simple Add-In that copies the value from the "dimension" User Parameter to the text in the 
# Sketch "text_sketch"
#
# With huge thanks to the guys on the Autodesk Forums...
#
#
# Fusion360 python adinn

import adsk.core, adsk.fusion, adsk.cam, traceback

_handlers = []
_app = adsk.core.Application.cast(None)
_ui = adsk.core.UserInterface.cast(None)
_last_dimension = 0.0
_prm_name = 'dimension'

class CommandHandler(adsk.core.ApplicationCommandEventHandler):
    def __init__(self, targetCommand):
        super().__init__()
        self.tgtCmd = targetCommand
    def notify(self, args):
        try:
            if _ui.activeCommand == self.tgtCmd:
                self.update_text_sketch()

        except:
            if _ui:
                _ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

    def update_text_sketch(self):
        try:
            des :adsk.fusion.Design = _app.activeProduct
            root :adsk.fusion.Component = des.rootComponent

            # parameter
            global _last_dimension, _prm_name
            prms :adsk.fusion.UserParameters = des.userParameters
            prm :adsk.fusion.UserParameter = prms.itemByName(_prm_name)

            if _last_dimension == prm.value:
                return

            # UnitsManager
            um :adsk.core.UnitsManager = des.unitsManager

            # skt
            skt :adsk.fusion.Sketch = root.sketches.itemByName('text_sketch')
            txt :adsk.fusion.SketchText = skt.sketchTexts.item(0)
            txt.text = '{:.1f}'.format(um.convert(prm.value, um.internalUnits, prm.unit))

            _last_dimension = prm.value

        except:
            if _ui:
                _ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))


def getTargetUserParamValue(prm_name :str):
    try:
        des :adsk.fusion.Design = _app.activeProduct
        prms :adsk.fusion.UserParameters = des.userParameters
        prm :adsk.fusion.UserParameter = prms.itemByName(prm_name)
        
        return prm.value
    except:
        if _ui:
            _ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))


def run(context):
    try:
        global _app, _ui
        _app = adsk.core.Application.get()
        _ui  = _app.userInterface

        onCommand = CommandHandler('ChangeParameterCommand')
        _ui.commandTerminated.add(onCommand)
        _handlers.append(onCommand)

        global _last_dimension, _prm_name
        _last_dimension = getTargetUserParamValue(_prm_name)
        print('--- Start addin ---')

    except:
        if _ui:
            _ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

def stop(context):
    try:
        for hdl in _handlers:
            _ui.commandTerminated.remove(hdl)
        _handlers.clear()
        print('--- Stop addin ---')

    except:
        if _ui:
            _ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))
